using System;
using NUnit.Framework;

namespace CSVParser
{
    [TestFixture]
    public class CsvReaderTests
    {
        [Test]
        public void ParsesEmptyString()
        {
            var data = CsvReader.Parse("");
            Assert.That(data.Rows.Count, Is.EqualTo(0));
        }

        [Test]
        public void OneRowPerLine()
        {
            //one header and 2 data rows
            var data = CsvReader.Parse("foo\nbar\nbaz");
            Assert.That(data.Rows.Count, Is.EqualTo(2));
        }

        [Test]
        public void ForgivingOfLineEndings()
        {
            var data = CsvReader.Parse("foo\nbar\rbaz\r\nmoo\n\rbaa\n\n\n\nwoof");
            Assert.That(data.Rows.Count, Is.EqualTo(5));
        }

        [Test]
        public void ColumnHeadingsSet()
        {
            var data = CsvReader.Parse("Foo,Bar");
            Assert.That(data.Columns.Count, Is.EqualTo(2));
            Assert.That(data.Columns["Foo"], Is.Not.Null);
            Assert.That(data.Columns[1].ColumnName, Is.EqualTo("Bar"));
        }

        [Test]
        public void DataAccessibleByIndexOrColumnName()
        {
            var data = CsvReader.Parse("Foo,Bar\na,b");
            var row = data.Rows[0];
            Assert.That(row[0], Is.EqualTo("a"));
            Assert.That(row["Bar"], Is.EqualTo("b"));
        }

        [Test]
        public void MismatchedDataRowThrowsException()
        {
            //This behavior is undefined, but I chose to throw an exception, because the data is bad.
            Assert.Throws<InvalidOperationException>(() => CsvReader.Parse("Foo,Bar \n one value only"));
            Assert.Throws<InvalidOperationException>(() => CsvReader.Parse("Foo,Bar \n a,b,c"));
        }

        [Test]
        public void QuotedValuesOk()
        {
            var data = CsvReader.Parse("Foo,Bar\n\"aaa\",b");
            var row = data.Rows[0];
            Assert.That(row["Foo"], Is.EqualTo("aaa"));
        }

        [Test]
        public void QuotesEscapeCommas()
        {
            var data = CsvReader.Parse("Foo,Bar\n\"aa,a\",b");
            var row = data.Rows[0];
            Assert.That(row["Foo"], Is.EqualTo("aa,a"));
        }

        [Test]
        public void QuotesEscapeQuotes()
        {
            var data = CsvReader.Parse("Foo,Bar\n\"aa\"\"a\",b");
            var row = data.Rows[0];
            Assert.That(row["Foo"], Is.EqualTo("aa\"a"));
        }

    }
}