using System;
using System.Collections.Generic;

namespace CSVParser
{
    internal class CsvTokenizer
    {
        private int _currentIndex;
        private readonly string _line;
        private bool _done;

        private CsvTokenizer(string line)
        {
            _line = line;
        }

        public static IList<string> TokenizeLine(string line)
        {
            return new CsvTokenizer(line).Tokenize();
        }

        private IList<string> Tokenize()
        {
            var list = new List<string>();
            _currentIndex = 0;
            while (!_done)
            {
                list.Add(GetNextToken());

            }
            return list;
        }

        private char GetNextChar()
        {
            return _line[_currentIndex++];
        }

        private char PeekNextChar()
        {
            return _line[_currentIndex];
        }

        private bool HasMoreChars()
        {
            return _currentIndex < _line.Length;
        }

        private string GetNextToken()
        {
            //beginning of token.
            if (!HasMoreChars())
            {
                //End of line is new token. Empty string.
                _done = true;
                return string.Empty;
            }
            char next = GetNextChar();
            if (next == ',')
            {
                //comma at beginning symbolizes empty token.
                return string.Empty;
            }
            else if (next == '"')
            {
                return GetQuotedToken();
            }
            else
            {
                return GetUnquotedToken(next);
            }
        }

        private string GetUnquotedToken(char start)
        {
            var token = start.ToString();
            while (HasMoreChars())
            {
                var next = GetNextChar();
                if (next == ',')
                {
                    return token;
                }
                if (next == '"')
                {
                    throw new InvalidOperationException("Fields with quotes must be escaped");
                }
                token += next;
            }
            _done = true;
            return token;
        }

        private string GetQuotedToken()
        {
            string token = string.Empty;
            bool ended = false;
            while (HasMoreChars())
            {
                char next = GetNextChar();
                if (next == '"')
                {
                    //check for escaped quote
                    if(HasMoreChars() && PeekNextChar() == '"')
                    {
                        GetNextChar(); //consume token
                        token += '"';
                        continue;
                    }
                    ended = true;
                    break;
                }
                token += next;
            }
            if (!ended)
            {
                throw new InvalidOperationException("Unterminated Quoted value");
            }
            //quotes are closed, consume whitespace until comma. or end of line
            while(HasMoreChars())
            {
                char next = GetNextChar();
                if(char.IsWhiteSpace(next))
                {
                    continue;
                }
                else if (next == ',')
                {
                    return token;
                }
                else
                {
                    throw new InvalidOperationException("No text allowed between quoted value and comma.");
                }
            }
            //if we get here it is because there are no more characters
            _done = true;
            return token;
        }
    }
}